@@ -5,6 +5,10 @@
 
 #include "player.h"
 
+typedef struct inter_threads_record {
+  uint32_t please_stop;
+} inter_threads_record;
+
 void *rtp_audio_receiver(void *arg);
 void *rtp_control_receiver(void *arg);
 void *rtp_timing_receiver(void *arg);
